#include "PCF8591.h"
#include <IRremote.h>

PCF8591 dev(0x48);

#define RECV_PIN 8
#define SGN_MAX 255
#define TIMER_1HZ 15625
#define MAX_OUTPUT_HISTORY 600

#define IR_BUTTON_0 22
#define IR_BUTTON_1 12
#define IR_BUTTON_2 24
#define IR_BUTTON_3 94
#define IR_BUTTON_4 8
#define IR_BUTTON_5 28
#define IR_BUTTON_6 90

char output_history[MAX_OUTPUT_HISTORY];
int output_history_last = 0;

char activeEffects[95] = { 0 };

float adjust_amplitude(float x) {
  if (x > 1.0f)
    return 1.0f;
  return x;
}

float process_signal(float x) {
  if (activeEffects[IR_BUTTON_1])
    x = apply_distortion(x);

  if (activeEffects[IR_BUTTON_2])
    x = apply_tremolo(x);

  if (activeEffects[IR_BUTTON_3])
    x = apply_flanger(x);

  if (activeEffects[IR_BUTTON_4])
    x = apply_echo(x);

  return adjust_amplitude(x);
}


float apply_distortion(float x) {
  float y = 1;

  if (x < 0.33f)
    y = 2.3 * x;
    
  else if (x < 0.66f)
    y = (3 - (2 - 3 * x) * (2 - 3 * x)) / 3;

  return y;
}

int index = 0;
float apply_tremolo(float x) {
  index++;
  float Fs = TIMER_1HZ / 8;
  float Fx = 5;
  float alpha = 0.25;
  float trem = (1.1 + alpha * sin(2 * M_PI * index * (Fx / Fs)));
  float y = trem * x;

  return y;
}

float flanger_delay = 0.04f;
float flanger_depth = 0.5f;
int index2 = 0;
float apply_flanger(float x) {
  index2++;
  float Fs = TIMER_1HZ;
  float rate = 1.0;
  float max_sample_delay = flanger_delay * MAX_OUTPUT_HISTORY;
  float current_sin = fabs(sin(2 * M_PI * index2 * (rate / Fs)));
  int current_delay = (int)ceil(current_sin * max_sample_delay);

  float out_delayed = (float)output_history[(output_history_last + (MAX_OUTPUT_HISTORY - 1) - current_delay) % MAX_OUTPUT_HISTORY];
  out_delayed /= SGN_MAX;

  return (1 - flanger_depth) * x + flanger_depth * out_delayed;
}


int echo_delay = 350;
float echo_depth = 0.4f;
float apply_echo(float x) {
  float out_delayed = (float)output_history[(output_history_last + (MAX_OUTPUT_HISTORY - 1) - echo_delay) % MAX_OUTPUT_HISTORY];
  out_delayed /= SGN_MAX;

  return (1 - echo_depth) * x + echo_depth * out_delayed;
}

void setup() {
  IrReceiver.begin(RECV_PIN);
  analogReference(INTERNAL);
  dev.begin();
  dev.enableDAC();
}

void loop() {
  unsigned int sensorValue = analogRead(A0);
  sensorValue = (sensorValue + 1) / 4 - 1;
  unsigned int out = process_signal(((float)sensorValue) / SGN_MAX) * SGN_MAX;

  output_history[output_history_last] = out;
  output_history_last = (output_history_last + 1) % MAX_OUTPUT_HISTORY;

  dev.analogWrite(out);

  if (IrReceiver.decode()) {
    IrReceiver.resume();
    int pressed_key = IrReceiver.decodedIRData.command;

    if (pressed_key != IR_BUTTON_0)
      activeEffects[pressed_key] = ~activeEffects[pressed_key];
    else
      memset(activeEffects, 0, sizeof(activeEffects));
  }
}
